@@ -43,6 +43,11 @@ extern const char             g_log_domain_gstreamer[];
 
 G_BEGIN_DECLS
 
+/* used by gstparse.c and grammar.y */
+struct _GstParseContext {
+  GList * missing_elements;
+};
+
 gboolean _priv_gst_in_valgrind (void);
 
 /* Initialize GStreamer private quark storage */
@@ -72,6 +77,17 @@ void _priv_gst_registry_cleanup (void);
 gboolean  priv_gst_structure_append_to_gstring (const GstStructure * structure,
                                                 GString            * s);
 
+/* registry cache backends */
+/* FIXME 0.11: use priv_ prefix */
+#ifdef USE_BINARY_REGISTRY
+gboolean 		gst_registry_binary_read_cache 	(GstRegistry * registry, const char *location);
+gboolean 		gst_registry_binary_write_cache	(GstRegistry * registry, const char *location);
+/* FIXME 0.11: this is in registry.h for backwards compatibility
+#else 
+gboolean 		gst_registry_xml_read_cache 	(GstRegistry * registry, const char *location);
+gboolean 		gst_registry_xml_write_cache 	(GstRegistry * registry, const char *location);
+*/
+#endif
 
 /*** debugging categories *****************************************************/
 
