@@ -54,16 +54,16 @@
  * interrupted with the gst_clock_id_unschedule() call. If the blocking wait is
  * unscheduled a return value of GST_CLOCK_UNSCHEDULED is returned.
  *
- * Periodic callbacks scheduled async will be repeadedly called automatically
+ * Periodic callbacks scheduled async will be repeatedly called automatically
  * until it is unscheduled. To schedule a sync periodic callback,
- * gst_clock_id_wait() should be called repeadedly.
+ * gst_clock_id_wait() should be called repeatedly.
  *
  * The async callbacks can happen from any thread, either provided by the core
  * or from a streaming thread. The application should be prepared for this.
  *
  * A #GstClockID that has been unscheduled cannot be used again for any wait
  * operation, a new #GstClockID should be created and the old unscheduled one
- * should be destroyed wirth gst_clock_id_unref().
+ * should be destroyed with gst_clock_id_unref().
  *
  * It is possible to perform a blocking wait on the same #GstClockID from
  * multiple threads. However, registering the same #GstClockID for multiple
@@ -82,9 +82,9 @@
  * running. Some clocks however do not progress when the element that provided
  * the clock is not PLAYING.
  *
- * When a clock has the GST_CLOCK_FLAG_CAN_SET_MASTER flag set, it can be
+ * When a clock has the #GST_CLOCK_FLAG_CAN_SET_MASTER flag set, it can be
  * slaved to another #GstClock with the gst_clock_set_master(). The clock will
- * then automatically be synchronized to this master clock by repeadedly
+ * then automatically be synchronized to this master clock by repeatedly
  * sampling the master clock and the slave clock and recalibrating the slave
  * clock with gst_clock_set_calibration(). This feature is mostly useful for
  * plugins that have an internal clock but must operate with another clock
@@ -116,14 +116,6 @@
 static GstAllocTrace *_gst_clock_entry_trace;
 #endif
 
-#if GLIB_CHECK_VERSION (2, 10, 0)
-#define ALLOC_ENTRY()     g_slice_new (GstClockEntry)
-#define FREE_ENTRY(entry) g_slice_free (GstClockEntry, entry)
-#else
-#define ALLOC_ENTRY()     g_new (GstClockEntry, 1)
-#define FREE_ENTRY(entry) g_free (entry)
-#endif
-
 /* #define DEBUGGING_ENABLED */
 
 #define DEFAULT_STATS                   FALSE
@@ -162,14 +154,14 @@ gst_clock_entry_new (GstClock * clock, GstClockTime time,
 {
   GstClockEntry *entry;
 
-  entry = ALLOC_ENTRY ();
+  entry = g_slice_new (GstClockEntry);
 #ifndef GST_DISABLE_TRACE
   gst_alloc_trace_new (_gst_clock_entry_trace, entry);
 #endif
   GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock,
       "created entry %p, time %" GST_TIME_FORMAT, entry, GST_TIME_ARGS (time));
 
-  gst_atomic_int_set (&entry->refcount, 1);
+  entry->refcount = 1;
   entry->clock = clock;
   entry->type = type;
   entry->time = time;
@@ -211,7 +203,7 @@ _gst_clock_id_free (GstClockID id)
 #ifndef GST_DISABLE_TRACE
   gst_alloc_trace_free (_gst_clock_entry_trace, id);
 #endif
-  FREE_ENTRY (id);
+  g_slice_free (GstClockEntry, id);
 }
 
 /**
@@ -266,8 +258,8 @@ gst_clock_new_single_shot_id (GstClock * clock, GstClockTime time)
  * @interval: the requested interval
  *
  * Get an ID from @clock to trigger a periodic notification.
- * The periodeic notifications will be start at time start_time and
- * will then be fired with the given interval. @id should be unreffed
+ * The periodic notifications will start at time @start_time and
+ * will then be fired with the given @interval. @id should be unreffed
  * after usage.
  *
  * Returns: A #GstClockID that can be used to request the time notification.
@@ -281,6 +273,7 @@ gst_clock_new_periodic_id (GstClock * clock, GstClockTime start_time,
   g_return_val_if_fail (GST_IS_CLOCK (clock), NULL);
   g_return_val_if_fail (GST_CLOCK_TIME_IS_VALID (start_time), NULL);
   g_return_val_if_fail (interval != 0, NULL);
+  g_return_val_if_fail (GST_CLOCK_TIME_IS_VALID (interval), NULL);
 
   return gst_clock_entry_new (clock,
       start_time, interval, GST_CLOCK_ENTRY_PERIODIC);
@@ -336,14 +329,14 @@ gst_clock_id_get_time (GstClockID id)
 /**
  * gst_clock_id_wait
  * @id: The #GstClockID to wait on
- * @jitter: A pointer that will contain the jitter, can be NULL.
+ * @jitter: A pointer that will contain the jitter, can be %NULL.
  *
  * Perform a blocking wait on @id. 
  * @id should have been created with gst_clock_new_single_shot_id()
  * or gst_clock_new_periodic_id() and should not have been unscheduled
  * with a call to gst_clock_id_unschedule(). 
  *
- * If the @jitter argument is not NULL and this function returns #GST_CLOCK_OK
+ * If the @jitter argument is not %NULL and this function returns #GST_CLOCK_OK
  * or #GST_CLOCK_EARLY, it will contain the difference
  * against the clock and the time of @id when this method was
  * called. 
@@ -412,9 +405,8 @@ gst_clock_id_wait (GstClockID id, GstClockTimeDiff * jitter)
   GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock,
       "done waiting entry %p, res: %d", id, res);
 
-  if (entry->type == GST_CLOCK_ENTRY_PERIODIC) {
-    entry->time += entry->interval;
-  }
+  if (entry->type == GST_CLOCK_ENTRY_PERIODIC)
+    entry->time = requested + entry->interval;
 
   if (clock->stats)
     gst_clock_update_stats (clock);
@@ -445,11 +437,11 @@ not_supported:
  * gst_clock_id_wait_async:
  * @id: a #GstClockID to wait on
  * @func: The callback function
- * @user_data: User data passed in the calback
+ * @user_data: User data passed in the callback
  *
  * Register a callback on the given #GstClockID @id with the given
  * function and user_data. When passing a #GstClockID with an invalid
- * time to this function, the callback will be called immediatly
+ * time to this function, the callback will be called immediately
  * with  a time set to GST_CLOCK_TIME_NONE. The callback will
  * be called when the time of @id has been reached.
  *
@@ -600,19 +592,21 @@ gst_clock_class_init (GstClockClass * klass)
   g_object_class_install_property (gobject_class, PROP_STATS,
       g_param_spec_boolean ("stats", "Stats",
           "Enable clock stats (unimplemented)", DEFAULT_STATS,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_WINDOW_SIZE,
       g_param_spec_int ("window-size", "Window size",
           "The size of the window used to calculate rate and offset", 2, 1024,
-          DEFAULT_WINDOW_SIZE, G_PARAM_READWRITE));
+          DEFAULT_WINDOW_SIZE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_WINDOW_THRESHOLD,
       g_param_spec_int ("window-threshold", "Window threshold",
           "The threshold to start calculating rate and offset", 2, 1024,
-          DEFAULT_WINDOW_THRESHOLD, G_PARAM_READWRITE));
+          DEFAULT_WINDOW_THRESHOLD,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_TIMEOUT,
       g_param_spec_uint64 ("timeout", "Timeout",
           "The amount of time, in nanoseconds, to sample master and slave clocks",
-          0, G_MAXUINT64, DEFAULT_TIMEOUT, G_PARAM_READWRITE));
+          0, G_MAXUINT64, DEFAULT_TIMEOUT,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -733,23 +727,47 @@ gst_clock_get_resolution (GstClock * clock)
  * @clock: a #GstClock to use
  * @internal: a clock time
  *
- * Converts the given @internal clock time to the real time, adjusting for the
+ * Converts the given @internal clock time to the external time, adjusting for the
  * rate and reference time set with gst_clock_set_calibration() and making sure
  * that the returned time is increasing. This function should be called with the
  * clock's OBJECT_LOCK held and is mainly used by clock subclasses.
  *
- * Returns: the converted time of the clock.
+ * This function is the reverse of gst_clock_unadjust_unlocked().
  *
- * MT safe.
+ * Returns: the converted time of the clock.
  */
 GstClockTime
 gst_clock_adjust_unlocked (GstClock * clock, GstClockTime internal)
 {
-  GstClockTime ret;
+  GstClockTime ret, cinternal, cexternal, cnum, cdenom;
 
-  ret = gst_util_uint64_scale (internal - clock->internal_calibration,
-      clock->rate_numerator, clock->rate_denominator);
-  ret += clock->external_calibration;
+  /* get calibration values for readability */
+  cinternal = clock->internal_calibration;
+  cexternal = clock->external_calibration;
+  cnum = clock->rate_numerator;
+  cdenom = clock->rate_denominator;
+
+  /* avoid divide by 0 */
+  if (cdenom == 0)
+    cnum = cdenom = 1;
+
+  /* The formula is (internal - cinternal) * cnum / cdenom + cexternal
+   *
+   * Since we do math on unsigned 64-bit ints we have to special case for
+   * internal < cinternal to get the sign right. this case is not very common,
+   * though.
+   */
+  if (G_LIKELY (internal >= cinternal)) {
+    ret = gst_util_uint64_scale (internal - cinternal, cnum, cdenom);
+    ret += cexternal;
+  } else {
+    ret = gst_util_uint64_scale (cinternal - internal, cnum, cdenom);
+    /* clamp to 0 */
+    if (cexternal > ret)
+      ret = cexternal - ret;
+    else
+      ret = 0;
+  }
 
   /* make sure the time is increasing */
   clock->last_time = MAX (ret, clock->last_time);
@@ -757,6 +775,51 @@ gst_clock_adjust_unlocked (GstClock * clock, GstClockTime internal)
   return clock->last_time;
 }
 
+/**
+ * gst_clock_unadjust_unlocked
+ * @clock: a #GstClock to use
+ * @external: an external clock time
+ *
+ * Converts the given @external clock time to the internal time of @clock,
+ * using the rate and reference time set with gst_clock_set_calibration().
+ * This function should be called with the clock's OBJECT_LOCK held and
+ * is mainly used by clock subclasses.
+ *
+ * This function is the reverse of gst_clock_adjust_unlocked().
+ *
+ * Returns: the internal time of the clock corresponding to @external.
+ *
+ * Since: 0.10.13
+ */
+GstClockTime
+gst_clock_unadjust_unlocked (GstClock * clock, GstClockTime external)
+{
+  GstClockTime ret, cinternal, cexternal, cnum, cdenom;
+
+  /* get calibration values for readability */
+  cinternal = clock->internal_calibration;
+  cexternal = clock->external_calibration;
+  cnum = clock->rate_numerator;
+  cdenom = clock->rate_denominator;
+
+  /* avoid divide by 0 */
+  if (cnum == 0)
+    cnum = cdenom = 1;
+
+  /* The formula is (external - cexternal) * cdenom / cnum + cinternal */
+  if (external >= cexternal) {
+    ret = gst_util_uint64_scale (external - cexternal, cdenom, cnum);
+    ret += cinternal;
+  } else {
+    ret = gst_util_uint64_scale (cexternal - external, cdenom, cnum);
+    if (cinternal > ret)
+      ret = cinternal - ret;
+    else
+      ret = 0;
+  }
+  return ret;
+}
+
 /**
  * gst_clock_get_internal_time
  * @clock: a #GstClock to query
@@ -765,7 +828,7 @@ gst_clock_adjust_unlocked (GstClock * clock, GstClockTime internal)
  * unadjusted for the offset and the rate.
  *
  * Returns: the internal time of the clock. Or GST_CLOCK_TIME_NONE when
- * giving wrong input.
+ * given invalid input.
  *
  * MT safe.
  */
@@ -807,7 +870,7 @@ not_supported:
  * offset and rate.
  *
  * Returns: the time of the clock. Or GST_CLOCK_TIME_NONE when
- * giving wrong input.
+ * given invalid input.
  *
  * MT safe.
  */
@@ -852,7 +915,7 @@ gst_clock_get_time (GstClock * clock)
  * follows:
  *
  * <programlisting>
- *   time = (internal_time - @internal) * @rate_num / @rate_denom + @external
+ *   time = (internal_time - internal) * rate_num / rate_denom + external
  * </programlisting>
  *
  * This formula is implemented in gst_clock_adjust_unlocked(). Of course, it
@@ -869,11 +932,17 @@ gst_clock_set_calibration (GstClock * clock, GstClockTime internal, GstClockTime
     external, GstClockTime rate_num, GstClockTime rate_denom)
 {
   g_return_if_fail (GST_IS_CLOCK (clock));
-  g_return_if_fail (rate_num >= 0);
-  g_return_if_fail (rate_denom > 0);
+  g_return_if_fail (rate_num != GST_CLOCK_TIME_NONE);
+  g_return_if_fail (rate_denom > 0 && rate_denom != GST_CLOCK_TIME_NONE);
   g_return_if_fail (internal <= gst_clock_get_internal_time (clock));
 
   GST_OBJECT_LOCK (clock);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock,
+      "internal %" GST_TIME_FORMAT " external %" GST_TIME_FORMAT " %"
+      G_GUINT64_FORMAT "/%" G_GUINT64_FORMAT " = %f", GST_TIME_ARGS (internal),
+      GST_TIME_ARGS (external), rate_num, rate_denom,
+      gst_guint64_to_gdouble (rate_num) / gst_guint64_to_gdouble (rate_denom));
+
   clock->internal_calibration = internal;
   clock->external_calibration = external;
   clock->rate_numerator = rate_num;
@@ -892,7 +961,7 @@ gst_clock_set_calibration (GstClock * clock, GstClockTime internal, GstClockTime
  * Gets the internal rate and reference time of @clock. See
  * gst_clock_set_calibration() for more information.
  *
- * @internal, @external, @rate_num, and @rate_denom can be left NULL if the
+ * @internal, @external, @rate_num, and @rate_denom can be left %NULL if the
  * caller is not interested in the values.
  *
  * MT safe.
@@ -915,7 +984,7 @@ gst_clock_get_calibration (GstClock * clock, GstClockTime * internal,
   GST_OBJECT_UNLOCK (clock);
 }
 
-/* will be called repeadedly to sample the master and slave clock
+/* will be called repeatedly to sample the master and slave clock
  * to recalibrate the clock  */
 static gboolean
 gst_clock_slave_callback (GstClock * master, GstClockTime time,
@@ -951,13 +1020,13 @@ gst_clock_slave_callback (GstClock * master, GstClockTime time,
  * A clock provider that slaves its clock to a master can get the current
  * calibration values with gst_clock_get_calibration().
  *
- * @master can be NULL in which case @clock will not be slaved anymore. It will
+ * @master can be %NULL in which case @clock will not be slaved anymore. It will
  * however keep reporting its time adjusted with the last configured rate 
  * and time offsets.
  *
- * Returns: TRUE if the clock is capable of being slaved to a master clock. 
+ * Returns: %TRUE if the clock is capable of being slaved to a master clock. 
  * Trying to set a master on a clock without the 
- * GST_CLOCK_FLAG_CAN_SET_MASTER flag will make this function return FALSE.
+ * #GST_CLOCK_FLAG_CAN_SET_MASTER flag will make this function return %FALSE.
  *
  * MT safe.
  */
@@ -974,7 +1043,8 @@ gst_clock_set_master (GstClock * clock, GstClock * master)
   if (master && !GST_OBJECT_FLAG_IS_SET (clock, GST_CLOCK_FLAG_CAN_SET_MASTER))
     goto not_supported;
 
-  GST_DEBUG_OBJECT (clock, "slaving to master clock %p", master);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock,
+      "slaving %p to master clock %p", clock, master);
   master_p = &clock->master;
   gst_object_replace ((GstObject **) master_p, (GstObject *) master);
   GST_OBJECT_UNLOCK (clock);
@@ -1002,7 +1072,8 @@ gst_clock_set_master (GstClock * clock, GstClock * master)
   /* ERRORS */
 not_supported:
   {
-    GST_DEBUG_OBJECT (clock, "cannot be slaved to a master clock");
+    GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock,
+        "cannot be slaved to a master clock");
     GST_OBJECT_UNLOCK (clock);
     return FALSE;
   }
@@ -1012,11 +1083,11 @@ not_supported:
  * gst_clock_get_master
  * @clock: a #GstClock 
  *
- * Get the master clock that @clock is slaved to or NULL when the clock is
+ * Get the master clock that @clock is slaved to or %NULL when the clock is
  * not slaved to any master clock.
  *
- * Returns: a master #GstClock or NULL when this clock is not slaved to a master
- * clock. Unref after usage.
+ * Returns: a master #GstClock or %NULL when this clock is not slaved to a
+ * master clock. Unref after usage.
  *
  * MT safe.
  */
@@ -1057,9 +1128,10 @@ do_linear_regression (GstClock * clock, GstClockTime * m_num,
   n = clock->filling ? clock->time_index : clock->window_size;
 
 #ifdef DEBUGGING_ENABLED
-  GST_DEBUG ("doing regression on:");
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "doing regression on:");
   for (i = j = 0; i < n; i++, j += 4)
-    GST_DEBUG ("  %" G_GUINT64_FORMAT "  %" G_GUINT64_FORMAT, x[j], y[j]);
+    GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock,
+        "  %" G_GUINT64_FORMAT "  %" G_GUINT64_FORMAT, x[j], y[j]);
 #endif
 
   xmin = ymin = G_MAXUINT64;
@@ -1069,8 +1141,10 @@ do_linear_regression (GstClock * clock, GstClockTime * m_num,
   }
 
 #ifdef DEBUGGING_ENABLED
-  GST_DEBUG ("min x: %" G_GUINT64_FORMAT, xmin);
-  GST_DEBUG ("min y: %" G_GUINT64_FORMAT, ymin);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "min x: %" G_GUINT64_FORMAT,
+      xmin);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "min y: %" G_GUINT64_FORMAT,
+      ymin);
 #endif
 
   newx = clock->times + 1;
@@ -1083,9 +1157,10 @@ do_linear_regression (GstClock * clock, GstClockTime * m_num,
   }
 
 #ifdef DEBUGGING_ENABLED
-  GST_DEBUG ("reduced numbers:");
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "reduced numbers:");
   for (i = j = 0; i < n; i++, j += 4)
-    GST_DEBUG ("  %" G_GUINT64_FORMAT "  %" G_GUINT64_FORMAT, newx[j], newy[j]);
+    GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock,
+        "  %" G_GUINT64_FORMAT "  %" G_GUINT64_FORMAT, newx[j], newy[j]);
 #endif
 
   /* have to do this precisely otherwise the results are pretty much useless.
@@ -1101,8 +1176,10 @@ do_linear_regression (GstClock * clock, GstClockTime * m_num,
   ybar /= n;
 
 #ifdef DEBUGGING_ENABLED
-  GST_DEBUG ("  xbar  = %" G_GUINT64_FORMAT, xbar);
-  GST_DEBUG ("  ybar  = %" G_GUINT64_FORMAT, ybar);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "  xbar  = %" G_GUINT64_FORMAT,
+      xbar);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "  ybar  = %" G_GUINT64_FORMAT,
+      ybar);
 #endif
 
   /* multiplying directly would give quantities on the order of 1e20 -> 60 bits;
@@ -1134,17 +1211,20 @@ do_linear_regression (GstClock * clock, GstClockTime * m_num,
   *r_squared = ((double) sxy * (double) sxy) / ((double) sxx * (double) syy);
 
 #ifdef DEBUGGING_ENABLED
-  GST_DEBUG ("  m      = %g", ((double) *m_num) / *m_denom);
-  GST_DEBUG ("  b      = %" G_GUINT64_FORMAT, *b);
-  GST_DEBUG ("  xbase  = %" G_GUINT64_FORMAT, *xbase);
-  GST_DEBUG ("  r2     = %g", *r_squared);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "  m      = %g",
+      ((double) *m_num) / *m_denom);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "  b      = %" G_GUINT64_FORMAT,
+      *b);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "  xbase  = %" G_GUINT64_FORMAT,
+      *xbase);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "  r2     = %g", *r_squared);
 #endif
 
   return TRUE;
 
 invalid:
   {
-    GST_DEBUG_OBJECT (clock, "sxx == 0, regression failed");
+    GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "sxx == 0, regression failed");
     return FALSE;
   }
 }
@@ -1162,12 +1242,12 @@ invalid:
  * observations and @clock is recalibrated.
  *
  * If this functions returns %TRUE, @r_squared will contain the 
- * correlation coefficient of the interpollation. A value of 1.0
+ * correlation coefficient of the interpolation. A value of 1.0
  * means a perfect regression was performed. This value can
  * be used to control the sampling frequency of the master and slave
  * clocks.
  *
- * Returns: TRUE if enough observations were added to run the 
+ * Returns: %TRUE if enough observations were added to run the 
  * regression algorithm.
  *
  * MT safe.
