@@ -25,7 +25,7 @@
  * @short_description: Abstract base class for all pipeline elements
  * @see_also: #GstElementFactory, #GstPad
  *
- * GstElement is the abstract base class needed to construct an element that 
+ * GstElement is the abstract base class needed to construct an element that
  * can be used in a GStreamer pipeline. Please refer to the plugin writers
  * guide for more information on creating #GstElement subclasses.
  *
@@ -82,6 +82,7 @@
 #include <gobject/gvaluecollector.h>
 
 #include "gstelement.h"
+#include "gstenumtypes.h"
 #include "gstbus.h"
 #include "gstmarshal.h"
 #include "gsterror.h"
@@ -118,8 +119,6 @@ static void gst_element_base_class_finalize (gpointer g_class);
 static void gst_element_dispose (GObject * object);
 static void gst_element_finalize (GObject * object);
 
-static GstStateChangeReturn gst_element_change_state (GstElement * element,
-    GstStateChange transition);
 static GstStateChangeReturn gst_element_change_state_func (GstElement * element,
     GstStateChange transition);
 static GstStateChangeReturn gst_element_get_state_func (GstElement * element,
@@ -133,6 +132,10 @@ static gboolean gst_element_default_send_event (GstElement * element,
 static gboolean gst_element_default_query (GstElement * element,
     GstQuery * query);
 
+static GstPadTemplate
+    * gst_element_class_get_request_pad_template (GstElementClass *
+    element_class, const gchar * name);
+
 #ifndef GST_DISABLE_LOADSAVE
 static xmlNodePtr gst_element_save_thyself (GstObject * object,
     xmlNodePtr parent);
@@ -188,7 +191,7 @@ gst_element_class_init (GstElementClass * klass)
   gst_element_signals[PAD_ADDED] =
       g_signal_new ("pad-added", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (GstElementClass, pad_added), NULL, NULL,
-      gst_marshal_VOID__OBJECT, G_TYPE_NONE, 1, G_TYPE_OBJECT);
+      gst_marshal_VOID__OBJECT, G_TYPE_NONE, 1, GST_TYPE_PAD);
   /**
    * GstElement::pad-removed:
    * @gstelement: the object which received the signal
@@ -199,7 +202,7 @@ gst_element_class_init (GstElementClass * klass)
   gst_element_signals[PAD_REMOVED] =
       g_signal_new ("pad-removed", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (GstElementClass, pad_removed), NULL, NULL,
-      gst_marshal_VOID__OBJECT, G_TYPE_NONE, 1, G_TYPE_OBJECT);
+      gst_marshal_VOID__OBJECT, G_TYPE_NONE, 1, GST_TYPE_PAD);
   /**
    * GstElement::no-more-pads:
    * @gstelement: the object which received the signal
@@ -236,6 +239,15 @@ gst_element_base_class_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
+  /* FIXME 0.11: Copy the element details and instead of clearing the
+   * pad template list copy the list and increase the refcount of
+   * the pad templates by one.
+   *
+   * This will make it possible to add pad templates and set element
+   * details in the class_init functions and is the real GObject way
+   * of doing things.
+   * See http://bugzilla.gnome.org/show_bug.cgi?id=491501
+   */
   memset (&element_class->details, 0, sizeof (GstElementDetails));
   element_class->padtemplates = NULL;
 }
@@ -254,6 +266,7 @@ static void
 gst_element_init (GstElement * element)
 {
   GST_STATE (element) = GST_STATE_NULL;
+  GST_STATE_TARGET (element) = GST_STATE_NULL;
   GST_STATE_NEXT (element) = GST_STATE_VOID_PENDING;
   GST_STATE_PENDING (element) = GST_STATE_VOID_PENDING;
   GST_STATE_RETURN (element) = GST_STATE_CHANGE_SUCCESS;
@@ -345,7 +358,7 @@ gst_element_requires_clock (GstElement * element)
  * @element: a #GstElement to query
  *
  * Query if the element provides a clock. A #GstClock provided by an
- * element can be used as the global #GstClock for the pipeline. 
+ * element can be used as the global #GstClock for the pipeline.
  * An element that can provide a clock is only required to do so in the PAUSED
  * state, this means when it is fully negotiated and has allocated the resources
  * to operate the clock.
@@ -370,7 +383,7 @@ gst_element_provides_clock (GstElement * element)
  * gst_element_provide_clock:
  * @element: a #GstElement to query
  *
- * Get the clock provided by the given element. 
+ * Get the clock provided by the given element.
  * <note>An element is only required to provide a clock in the PAUSED
  * state. Some elements can provide a clock in other states.</note>
  *
@@ -421,7 +434,7 @@ gst_element_set_clock (GstElement * element, GstClock * clock)
 
   oclass = GST_ELEMENT_GET_CLASS (element);
 
-  GST_DEBUG_OBJECT (element, "setting clock %p", clock);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, element, "setting clock %p", clock);
 
   if (oclass->set_clock)
     res = oclass->set_clock (element, clock);
@@ -474,14 +487,18 @@ gst_element_get_clock (GstElement * element)
 void
 gst_element_set_base_time (GstElement * element, GstClockTime time)
 {
+  GstClockTime old;
+
   g_return_if_fail (GST_IS_ELEMENT (element));
 
   GST_OBJECT_LOCK (element);
+  old = element->base_time;
   element->base_time = time;
   GST_OBJECT_UNLOCK (element);
 
-  GST_DEBUG_OBJECT (element, "set base_time=%" GST_TIME_FORMAT,
-      GST_TIME_ARGS (time));
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, element,
+      "set base_time=%" GST_TIME_FORMAT ", old %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (time), GST_TIME_ARGS (old));
 }
 
 /**
@@ -898,7 +915,7 @@ gst_element_request_pad (GstElement * element, GstPadTemplate * templ,
  * @name: the name of the request #GstPad to retrieve.
  *
  * Retrieves a pad from the element by name. This version only retrieves
- * request pads. The pad should be released with 
+ * request pads. The pad should be released with
  * gst_element_release_request_pad().
  *
  * Returns: requested #GstPad if found, otherwise %NULL. Release after usage.
@@ -921,22 +938,31 @@ gst_element_get_request_pad (GstElement * element, const gchar * name)
 
   class = GST_ELEMENT_GET_CLASS (element);
 
+  /* if the name contains a %, we assume it's the complete template name. Get
+   * the template and try to get a pad */
   if (strstr (name, "%")) {
-    templ = gst_element_class_get_pad_template (class, name);
+    templ = gst_element_class_get_request_pad_template (class, name);
     req_name = NULL;
     if (templ)
       templ_found = TRUE;
   } else {
+    /* there is no % in the name, try to find a matching template */
     list = gst_element_class_get_pad_template_list (class);
     while (!templ_found && list) {
       templ = (GstPadTemplate *) list->data;
       if (templ->presence == GST_PAD_REQUEST) {
+        GST_CAT_DEBUG (GST_CAT_PADS, "comparing %s to %s", name,
+            templ->name_template);
+        /* see if we find an exact match */
+        if (strcmp (name, templ->name_template) == 0) {
+          templ_found = TRUE;
+          req_name = name;
+          break;
+        }
         /* Because of sanity checks in gst_pad_template_new(), we know that %s
            and %d, occurring at the end of the name_template, are the only
            possibilities. */
-        GST_CAT_DEBUG (GST_CAT_PADS, "comparing %s to %s", name,
-            templ->name_template);
-        if ((str = strchr (templ->name_template, '%'))
+        else if ((str = strchr (templ->name_template, '%'))
             && strncmp (templ->name_template, name,
                 str - templ->name_template) == 0
             && strlen (name) > str - templ->name_template) {
@@ -977,13 +1003,15 @@ gst_element_get_request_pad (GstElement * element, const gchar * name)
  * Retrieves a pad from @element by name. Tries gst_element_get_static_pad()
  * first, then gst_element_get_request_pad().
  *
- * <note>Usage of this function is not recommended as it is unclear if the reference
+ * Deprecated: This function is deprecated as it's unclear if the reference
  * to the result pad should be released with gst_object_unref() in case of a static pad
- * or gst_element_release_request_pad() in case of a request pad.</note>
+ * or gst_element_release_request_pad() in case of a request pad.
+ * Use gst_element_get_static_pad() or gst_element_get_request_pad() instead.
  *
  * Returns: the #GstPad if found, otherwise %NULL. Unref or Release after usage,
  * depending on the type of the pad.
  */
+#ifndef GST_REMOVE_DEPRECATED
 GstPad *
 gst_element_get_pad (GstElement * element, const gchar * name)
 {
@@ -998,6 +1026,7 @@ gst_element_get_pad (GstElement * element, const gchar * name)
 
   return pad;
 }
+#endif /* GST_REMOVE_DEPRECATED */
 
 static GstIteratorItem
 iterate_pad (GstIterator * it, GstPad * pad)
@@ -1095,6 +1124,12 @@ gst_element_class_add_pad_template (GstElementClass * klass,
   g_return_if_fail (GST_IS_ELEMENT_CLASS (klass));
   g_return_if_fail (GST_IS_PAD_TEMPLATE (templ));
 
+  /* FIXME 0.11: allow replacing the pad templates by
+   * calling this with the same name as an already existing pad
+   * template. For this we _must_ _not_ ref the added pad template
+   * a second time and _must_ document that this function takes
+   * ownership of the pad template. Otherwise we will leak pad templates
+   * or the caller unref's the pad template and it disappears */
   /* avoid registering pad templates with the same name */
   g_return_if_fail (gst_element_class_get_pad_template (klass,
           templ->name_template) == NULL);
@@ -1124,6 +1159,41 @@ gst_element_class_set_details (GstElementClass * klass,
   __gst_element_details_copy (&klass->details, details);
 }
 
+/**
+ * gst_element_class_set_details_simple:
+ * @klass: class to set details for
+ * @longname: The long English name of the element. E.g. "File Sink"
+ * @classification: String describing the type of element, as an unordered list
+ * separated with slashes ('/'). See draft-klass.txt of the design docs
+ * for more details and common types. E.g: "Sink/File"
+ * @description: Sentence describing the purpose of the element. 
+ * E.g: "Write stream to a file"
+ * @author: Name and contact details of the author(s). Use \n to separate 
+ * multiple author details. E.g: "Joe Bloggs &lt;joe.blogs at foo.com&gt;"
+ *
+ * Sets the detailed information for a #GstElementClass. Simpler version of 
+ * gst_element_class_set_details() that generates less linker overhead.
+ * <note>This function is for use in _base_init functions only.</note>
+ *
+ * The detail parameter strings are copied into the #GstElementDetails for
+ * the element class.
+ *
+ * Since: 0.10.14
+ */
+void
+gst_element_class_set_details_simple (GstElementClass * klass,
+    const gchar * longname, const gchar * classification,
+    const gchar * description, const gchar * author)
+{
+  const GstElementDetails details =
+      GST_ELEMENT_DETAILS ((gchar *) longname, (gchar *) classification,
+      (gchar *) description, (gchar *) author);
+
+  g_return_if_fail (GST_IS_ELEMENT_CLASS (klass));
+
+  __gst_element_details_copy (&klass->details, &details);
+}
+
 /**
  * gst_element_class_get_pad_template_list:
  * @element_class: a #GstElementClass to get pad templates of.
@@ -1180,6 +1250,19 @@ gst_element_class_get_pad_template (GstElementClass * element_class,
   return NULL;
 }
 
+static GstPadTemplate *
+gst_element_class_get_request_pad_template (GstElementClass * element_class,
+    const gchar * name)
+{
+  GstPadTemplate *tmpl;
+
+  tmpl = gst_element_class_get_pad_template (element_class, name);
+  if (tmpl != NULL && tmpl->presence == GST_PAD_REQUEST)
+    return tmpl;
+
+  return NULL;
+}
+
 /* get a random pad on element of the given direction.
  * The pad is random in a sense that it is the first pad that is (optionaly) linked.
  */
@@ -1275,7 +1358,7 @@ gst_element_default_send_event (GstElement * element, GstEvent * event)
  *
  * Sends an event to an element. If the element doesn't implement an
  * event handler, the event will be pushed on a random linked sink pad for
- * upstream events or a random linked source pad for downstream events. 
+ * upstream events or a random linked source pad for downstream events.
  *
  * This function takes owership of the provided event so you should
  * gst_event_ref() it if you want to reuse the event after this call.
@@ -1295,6 +1378,7 @@ gst_element_send_event (GstElement * element, GstEvent * event)
 
   oclass = GST_ELEMENT_GET_CLASS (element);
 
+  GST_STATE_LOCK (element);
   if (oclass->send_event) {
     GST_CAT_DEBUG (GST_CAT_ELEMENT_PADS, "send %s event on element %s",
         GST_EVENT_TYPE_NAME (event), GST_ELEMENT_NAME (element));
@@ -1302,6 +1386,8 @@ gst_element_send_event (GstElement * element, GstEvent * event)
   } else {
     result = gst_element_default_send_event (element, event);
   }
+  GST_STATE_UNLOCK (element);
+
   return result;
 }
 
@@ -1488,7 +1574,8 @@ gst_element_post_message (GstElement * element, GstMessage * message)
   /* ERRORS */
 no_bus:
   {
-    GST_DEBUG_OBJECT (element, "not posting message %p: no bus", message);
+    GST_CAT_DEBUG_OBJECT (GST_CAT_MESSAGE, element,
+        "not posting message %p: no bus", message);
     GST_OBJECT_UNLOCK (element);
     gst_message_unref (message);
     return FALSE;
@@ -1710,38 +1797,51 @@ gboolean
 gst_element_sync_state_with_parent (GstElement * element)
 {
   GstElement *parent;
+  GstState target;
+  GstStateChangeReturn ret;
 
   g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
 
   if ((parent = GST_ELEMENT_CAST (gst_element_get_parent (element)))) {
     GstState parent_current, parent_pending;
-    GstStateChangeReturn ret;
 
     GST_OBJECT_LOCK (parent);
     parent_current = GST_STATE (parent);
     parent_pending = GST_STATE_PENDING (parent);
     GST_OBJECT_UNLOCK (parent);
 
-    GST_CAT_DEBUG (GST_CAT_STATES,
-        "syncing state of element %s (%s) to %s (%s, %s)",
-        GST_ELEMENT_NAME (element),
+    /* set to pending if there is one, else we set it to the current state of
+     * the parent */
+    if (parent_pending != GST_STATE_VOID_PENDING)
+      target = parent_pending;
+    else
+      target = parent_current;
+
+    GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element,
+        "syncing state (%s) to parent %s %s (%s, %s)",
         gst_element_state_get_name (GST_STATE (element)),
-        GST_ELEMENT_NAME (parent), gst_element_state_get_name (parent_current),
+        GST_ELEMENT_NAME (parent), gst_element_state_get_name (target),
+        gst_element_state_get_name (parent_current),
         gst_element_state_get_name (parent_pending));
 
-    ret = gst_element_set_state (element, parent_current);
+    ret = gst_element_set_state (element, target);
     if (ret == GST_STATE_CHANGE_FAILURE)
       goto failed;
 
     gst_object_unref (parent);
 
     return TRUE;
+  } else {
+    GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element, "element has no parent");
   }
   return FALSE;
 
   /* ERROR */
 failed:
   {
+    GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element,
+        "syncing state failed (%s)",
+        gst_element_state_change_return_get_name (ret));
     return FALSE;
   }
 }
@@ -1816,11 +1916,11 @@ gst_element_get_state_func (GstElement * element,
         ret = GST_STATE_CHANGE_FAILURE;
       }
     }
-  }
-  /* if nothing is pending anymore we can return SUCCESS */
-  if (GST_STATE_PENDING (element) == GST_STATE_VOID_PENDING) {
-    GST_CAT_LOG_OBJECT (GST_CAT_STATES, element, "nothing pending");
-    ret = GST_STATE_CHANGE_SUCCESS;
+    /* if nothing is pending anymore we can return SUCCESS */
+    if (GST_STATE_PENDING (element) == GST_STATE_VOID_PENDING) {
+      GST_CAT_LOG_OBJECT (GST_CAT_STATES, element, "nothing pending");
+      ret = GST_STATE_CHANGE_SUCCESS;
+    }
   }
 
 done:
@@ -1845,7 +1945,7 @@ interrupted:
     if (pending)
       *pending = GST_STATE_VOID_PENDING;
 
-    GST_CAT_INFO_OBJECT (GST_CAT_STATES, element, "get_state() interruped");
+    GST_CAT_INFO_OBJECT (GST_CAT_STATES, element, "interruped");
 
     GST_OBJECT_UNLOCK (element);
 
@@ -1961,37 +2061,37 @@ nothing_aborted:
 }
 
 /**
- * gst_element_continue_state:   
- * @element: a #GstElement to continue the state change of.      
+ * gst_element_continue_state:
+ * @element: a #GstElement to continue the state change of.
  * @ret: The previous state return value
- *       
- * Commit the state change of the element and proceed to the next 
- * pending state if any. This function is used   
- * by elements that do asynchronous state changes.       
- * The core will normally call this method automatically when an         
- * element returned %GST_STATE_CHANGE_SUCCESS from the state change function.      
  *
- * If after calling this method the element still has not reached        
- * the pending state, the next state change is performed.        
- *       
+ * Commit the state change of the element and proceed to the next
+ * pending state if any. This function is used
+ * by elements that do asynchronous state changes.
+ * The core will normally call this method automatically when an
+ * element returned %GST_STATE_CHANGE_SUCCESS from the state change function.
+ *
+ * If after calling this method the element still has not reached
+ * the pending state, the next state change is performed.
+ *
  * This method is used internally and should normally not be called by plugins
  * or applications.
- *       
- * Returns: The result of the commit state change.       
- *       
- * MT safe.      
+ *
+ * Returns: The result of the commit state change.
+ *
+ * MT safe.
  */
 GstStateChangeReturn
 gst_element_continue_state (GstElement * element, GstStateChangeReturn ret)
 {
-  GstState pending;
-  GstState old_ret, old_state, old_next;
-  GstState current, next;
+  GstStateChangeReturn old_ret;
+  GstState old_state, old_next;
+  GstState current, next, pending;
   GstMessage *message;
   GstStateChange transition;
 
   GST_OBJECT_LOCK (element);
-  old_ret = (GstState) GST_STATE_RETURN (element);
+  old_ret = GST_STATE_RETURN (element);
   GST_STATE_RETURN (element) = ret;
   pending = GST_STATE_PENDING (element);
 
@@ -2047,18 +2147,24 @@ complete:
     GST_STATE_PENDING (element) = GST_STATE_VOID_PENDING;
     GST_STATE_NEXT (element) = GST_STATE_VOID_PENDING;
 
-    GST_CAT_INFO_OBJECT (GST_CAT_STATES, element, "completed state change");
+    GST_CAT_INFO_OBJECT (GST_CAT_STATES, element,
+        "completed state change to %s", gst_element_state_get_name (pending));
     GST_OBJECT_UNLOCK (element);
 
     /* don't post silly messages with the same state. This can happen
      * when an element state is changed to what it already was. For bins
      * this can be the result of a lost state, which we check with the
-     * previous return value. 
-     * We do signal the cond though as a _get_state() might be blocking 
+     * previous return value.
+     * We do signal the cond though as a _get_state() might be blocking
      * on it. */
     if (old_state != old_next || old_ret == GST_STATE_CHANGE_ASYNC) {
-      message = gst_message_new_state_changed (GST_OBJECT_CAST (element),
-          old_state, old_next, GST_STATE_VOID_PENDING);
+      GST_CAT_INFO_OBJECT (GST_CAT_STATES, element,
+          "posting state-changed %s to %s",
+          gst_element_state_get_name (old_state),
+          gst_element_state_get_name (old_next));
+      message =
+          gst_message_new_state_changed (GST_OBJECT_CAST (element), old_state,
+          old_next, GST_STATE_VOID_PENDING);
       gst_element_post_message (element, message);
     }
 
@@ -2076,10 +2182,16 @@ complete:
  * element is copied to the pending state so that any call to
  * gst_element_get_state() will return %GST_STATE_CHANGE_ASYNC.
  *
+ * An ASYNC_START message is posted with an indication to distribute a new
+ * base_time to the element.
+ * If the element was PLAYING, it will go to PAUSED. The element
+ * will be restored to its PLAYING state by the parent pipeline when it
+ * prerolls again.
+ *
  * This is mostly used for elements that lost their preroll buffer
- * in the %GST_STATE_PAUSED state after a flush, they become %GST_STATE_PAUSED
- * again if a new preroll buffer is queued.
- * This function can only be called when the element is currently
+ * in the %GST_STATE_PAUSED or %GST_STATE_PLAYING state after a flush,
+ * they will go to their pending state again when a new preroll buffer is
+ * queued. This function can only be called when the element is currently
  * not in error or an async state change.
  *
  * This function is used internally and should normally not be called from
@@ -2090,7 +2202,7 @@ complete:
 void
 gst_element_lost_state (GstElement * element)
 {
-  GstState current_state;
+  GstState old_state, new_state;
   GstMessage *message;
 
   g_return_if_fail (GST_IS_ELEMENT (element));
@@ -2100,22 +2212,31 @@ gst_element_lost_state (GstElement * element)
       GST_STATE_RETURN (element) == GST_STATE_CHANGE_FAILURE)
     goto nothing_lost;
 
-  current_state = GST_STATE (element);
+  old_state = GST_STATE (element);
+
+  /* when we were PLAYING, the new state is PAUSED. We will also not
+   * automatically go to PLAYING but let the parent bin(s) set us to PLAYING
+   * when we preroll. */
+  if (old_state > GST_STATE_PAUSED)
+    new_state = GST_STATE_PAUSED;
+  else
+    new_state = old_state;
 
   GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element,
-      "lost state of %s", gst_element_state_get_name (current_state));
+      "lost state of %s to %s", gst_element_state_get_name (old_state),
+      gst_element_state_get_name (new_state));
 
-  GST_STATE_NEXT (element) = current_state;
-  GST_STATE_PENDING (element) = current_state;
+  GST_STATE (element) = new_state;
+  GST_STATE_NEXT (element) = new_state;
+  GST_STATE_PENDING (element) = new_state;
   GST_STATE_RETURN (element) = GST_STATE_CHANGE_ASYNC;
   GST_OBJECT_UNLOCK (element);
 
   message = gst_message_new_state_changed (GST_OBJECT_CAST (element),
-      current_state, current_state, current_state);
+      new_state, new_state, new_state);
   gst_element_post_message (element, message);
 
-  /* and mark us dirty */
-  message = gst_message_new_state_dirty (GST_OBJECT_CAST (element));
+  message = gst_message_new_async_start (GST_OBJECT_CAST (element), TRUE);
   gst_element_post_message (element, message);
 
   return;
@@ -2164,7 +2285,7 @@ gst_element_set_state (GstElement * element, GstState state)
 
 /*
  * default set state function, calculates the next state based
- * on current state and calls the change_state function 
+ * on current state and calls the change_state function
  */
 static GstStateChangeReturn
 gst_element_set_state_func (GstElement * element, GstState state)
@@ -2179,7 +2300,7 @@ gst_element_set_state_func (GstElement * element, GstState state)
   GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element, "set_state to %s",
       gst_element_state_get_name (state));
 
-  /* state lock is taken to protect the set_state() and get_state() 
+  /* state lock is taken to protect the set_state() and get_state()
    * procedures, it does not lock any variables. */
   GST_STATE_LOCK (element);
 
@@ -2197,10 +2318,15 @@ gst_element_set_state_func (GstElement * element, GstState state)
   current = GST_STATE (element);
   next = GST_STATE_NEXT (element);
   old_pending = GST_STATE_PENDING (element);
-  /* increment state cookie so that we can track each state change */
-  element->state_cookie++;
 
-  /* this is the (new) state we should go to */
+  /* this is the (new) state we should go to. TARGET is the last state we set on
+   * the element. */
+  if (state != GST_STATE_TARGET (element)) {
+    GST_STATE_TARGET (element) = state;
+    /* increment state cookie so that we can track each state change. We only do
+     * this if this is actually a new state change. */
+    element->state_cookie++;
+  }
   GST_STATE_PENDING (element) = state;
 
   GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element,
@@ -2244,7 +2370,7 @@ gst_element_set_state_func (GstElement * element, GstState state)
       (next != state ? "intermediate" : "final"),
       gst_element_state_get_name (current), gst_element_state_get_name (next));
 
-  /* now signal any waiters, they will error since the cookie was increased */
+  /* now signal any waiters, they will error since the cookie was incremented */
   GST_STATE_BROADCAST (element);
 
   GST_OBJECT_UNLOCK (element);
@@ -2271,8 +2397,19 @@ was_busy:
   }
 }
 
-/* with STATE_LOCK */
-static GstStateChangeReturn
+/**
+ * gst_element_change_state:
+ * @element: a #GstElement
+ * @transition: the requested transition
+ *
+ * Perform @transition on @element.
+ *
+ * This function must be called with STATE_LOCK held and is mainly used
+ * internally.
+ *
+ * Returns: the #GstStateChangeReturn of the state transition.
+ */
+GstStateChangeReturn
 gst_element_change_state (GstElement * element, GstStateChange transition)
 {
   GstElementClass *oclass;
@@ -2300,22 +2437,26 @@ gst_element_change_state (GstElement * element, GstStateChange transition)
       gst_element_abort_state (element);
       break;
     case GST_STATE_CHANGE_ASYNC:
+    {
+      GstState target;
+
       GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element,
           "element will change state ASYNC");
 
-      /* if we go upwards, we give the app a change to wait for
-       * completion */
-      if (current < next)
+      target = GST_STATE_TARGET (element);
+
+      if (target > GST_STATE_READY)
         goto async;
 
       /* else we just continue the state change downwards */
       GST_CAT_INFO_OBJECT (GST_CAT_STATES, element,
-          "forcing commit state %s < %s",
-          gst_element_state_get_name (current),
-          gst_element_state_get_name (next));
+          "forcing commit state %s <= %s",
+          gst_element_state_get_name (target),
+          gst_element_state_get_name (GST_STATE_READY));
 
       ret = gst_element_continue_state (element, GST_STATE_CHANGE_SUCCESS);
       break;
+    }
     case GST_STATE_CHANGE_SUCCESS:
       GST_CAT_DEBUG_OBJECT (GST_CAT_STATES, element,
           "element changed state SUCCESS");
@@ -2411,7 +2552,7 @@ iterator_activate_fold_with_resync (GstIterator * iter,
         /* all pads iterated, return collected value */
         goto done;
       default:
-        /* iterator returned _ERROR or premature end with _OK, 
+        /* iterator returned _ERROR or premature end with _OK,
          * mark an error and exit */
         g_value_set_boolean (&ret, FALSE);
         goto done;
@@ -2424,7 +2565,7 @@ done:
 
 /* is called with STATE_LOCK
  *
- * Pads are activated from source pads to sinkpads. 
+ * Pads are activated from source pads to sinkpads.
  */
 static gboolean
 gst_element_pads_activate (GstElement * element, gboolean active)
@@ -2432,7 +2573,8 @@ gst_element_pads_activate (GstElement * element, gboolean active)
   GstIterator *iter;
   gboolean res;
 
-  GST_DEBUG_OBJECT (element, "pads_activate with active %d", active);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_ELEMENT_PADS, element,
+      "pads_activate with active %d", active);
 
   iter = gst_element_iterate_src_pads (element);
   res =
@@ -2461,7 +2603,8 @@ gst_element_pads_activate (GstElement * element, gboolean active)
       goto caps_failed;
   }
 
-  GST_DEBUG_OBJECT (element, "pads_activate successful");
+  GST_CAT_DEBUG_OBJECT (GST_CAT_ELEMENT_PADS, element,
+      "pads_activate successful");
 
   return TRUE;
 
@@ -2581,9 +2724,8 @@ gst_element_dispose (GObject * object)
   if (GST_STATE (element) != GST_STATE_NULL)
     goto not_null;
 
-  g_return_if_fail (GST_STATE_PENDING (element) == GST_STATE_VOID_PENDING);
-
-  GST_DEBUG_OBJECT (element, "removing %d pads", g_list_length (element->pads));
+  GST_CAT_DEBUG_OBJECT (GST_CAT_ELEMENT_PADS, element,
+      "removing %d pads", g_list_length (element->pads));
   /* first we break all our links with the outside */
   while (element->pads && element->pads->data) {
     /* don't call _remove_pad with NULL */
@@ -2818,7 +2960,8 @@ gst_element_set_bus (GstElement * element, GstBus * bus)
  * gst_element_get_bus:
  * @element: a #GstElement to get the bus of.
  *
- * Returns the bus of the element.
+ * Returns the bus of the element. Note that only a #GstPipeline will provide a
+ * bus for the application.
  *
  * Returns: the element's #GstBus. unref after usage.
  *
@@ -2836,7 +2979,8 @@ gst_element_get_bus (GstElement * element)
     gst_object_ref (result);
   GST_OBJECT_UNLOCK (element);
 
-  GST_DEBUG_OBJECT (element, "got bus %" GST_PTR_FORMAT, result);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_BUS, element, "got bus %" GST_PTR_FORMAT,
+      result);
 
   return result;
 }
