@@ -77,9 +77,9 @@ G_BEGIN_DECLS
  * @GST_STATE_CHANGE_FAILURE   : the state change failed
  * @GST_STATE_CHANGE_SUCCESS   : the state change succeeded
  * @GST_STATE_CHANGE_ASYNC     : the state change will happen asynchronously
- * @GST_STATE_CHANGE_NO_PREROLL: the state change succeeded but the element cannot
- *                               produce data in PAUSED. This typically happens
- *                               with live sources.
+ * @GST_STATE_CHANGE_NO_PREROLL: the state change succeeded but the element
+ *                               cannot produce data in PAUSED. This typically
+ *                               happens with live sources.
  *
  * The possible return values from a state change function. Only
  * @GST_STATE_CHANGE_FAILURE is a real failure.
@@ -531,9 +531,13 @@ struct _GstElementClass
 /* element class pad templates */
 void			gst_element_class_add_pad_template	(GstElementClass *klass, GstPadTemplate *templ);
 GstPadTemplate*		gst_element_class_get_pad_template	(GstElementClass *element_class, const gchar *name);
-GList*                  gst_element_class_get_pad_template_list (GstElementClass *element_class);
-void			gst_element_class_set_details		(GstElementClass *klass,
-								 const GstElementDetails *details);
+GList*                  gst_element_class_get_pad_template_list	(GstElementClass *element_class);
+void			gst_element_class_set_details		(GstElementClass *klass, const GstElementDetails *details);
+void			gst_element_class_set_details_simple	(GstElementClass *klass,
+			                                    	 const gchar     *longname,
+			                                    	 const gchar     *classification,
+			                                    	 const gchar     *description,
+			                                         const gchar     *author);
 
 /* element instance */
 GType			gst_element_get_type		(void);
@@ -597,7 +601,9 @@ gboolean		gst_element_add_pad		(GstElement *element, GstPad *pad);
 gboolean		gst_element_remove_pad		(GstElement *element, GstPad *pad);
 void			gst_element_no_more_pads	(GstElement *element);
 
+#ifndef GST_DISABLE_DEPRECATED
 GstPad*			gst_element_get_pad		(GstElement *element, const gchar *name);
+#endif /* GST_DISABLE_DEPRECATED */
 GstPad*			gst_element_get_static_pad	(GstElement *element, const gchar *name);
 GstPad*			gst_element_get_request_pad	(GstElement *element, const gchar *name);
 void			gst_element_release_request_pad	(GstElement *element, GstPad *pad);
