@@ -44,7 +44,9 @@
  * processing.  In short, an element doesn't know what it is being used for.
  *
  * It is the application or compound element using the given element that
- * has more context about the use of the element.  The thrown errors should
+ * has more context about the use of the element. Errors can be received by
+ * listening to the #GstBus of the element/pipeline for #GstMessage objects with
+ * the type %GST_MESSAGE_ERROR or %GST_MESSAGE_WARNING. The thrown errors should
  * be inspected, and filtered if appropriate.
  *
  * An application is expected to, by default, present the user with a
@@ -92,6 +94,8 @@
  *     passed.  #GST_ERROR_SYSTEM can be used as a shortcut to give
  *     debug information on a system call error.</para></listitem>
  * </itemizedlist>
+ *
+ * Last reviewed on 2006-09-15 (0.10.10)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -156,6 +160,9 @@ _gst_core_errors_init (void)
       N_("Your GStreamer installation is missing a plug-in."));
   TABLE (t, CORE, CLOCK,
       N_("Internal GStreamer error: clock problem." FILE_A_BUG));
+  TABLE (t, CORE, DISABLED,
+      N_("This application is trying to use GStreamer functionality that "
+          "has been disabled."));
 
   return t;
 }
@@ -175,7 +182,7 @@ _gst_library_errors_init (void)
           "to this error." FILE_A_BUG));
   TABLE (t, LIBRARY, INIT, N_("Could not initialize supporting library."));
   TABLE (t, LIBRARY, SHUTDOWN, N_("Could not close supporting library."));
-  TABLE (t, LIBRARY, SETTINGS, N_("Could not close supporting library."));
+  TABLE (t, LIBRARY, SETTINGS, N_("Could not configure supporting library."));
 
   return t;
 }
@@ -237,6 +244,11 @@ _gst_stream_errors_init (void)
   TABLE (t, STREAM, DEMUX, N_("Could not demultiplex stream."));
   TABLE (t, STREAM, MUX, N_("Could not multiplex stream."));
   TABLE (t, STREAM, FORMAT, N_("The stream is in the wrong format."));
+  TABLE (t, STREAM, DECRYPT,
+      N_("The stream is encrypted and decryption is not supported."));
+  TABLE (t, STREAM, DECRYPT_NOKEY,
+      N_("The stream is encrypted and can't be decrypted because no suitable "
+          "key has been supplied."));
 
   return t;
 }
