@@ -48,6 +48,9 @@
 #include "gstinfo.h"
 
 #include "gstghostpad.h"
+#include "gst.h"
+
+#define GST_CAT_DEFAULT GST_CAT_PADS
 
 #define GST_TYPE_PROXY_PAD              (gst_proxy_pad_get_type ())
 #define GST_IS_PROXY_PAD(obj)           (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_PROXY_PAD))
@@ -84,6 +87,7 @@ struct _GstProxyPadClass
   gpointer _gst_reserved[1];
 };
 
+static GType gst_proxy_pad_get_type (void);
 
 G_DEFINE_TYPE (GstProxyPad, gst_proxy_pad, GST_TYPE_PAD);
 
@@ -116,7 +120,7 @@ gst_proxy_pad_class_init (GstProxyPadClass * klass)
 #endif
 }
 
-const GstQueryType *
+static const GstQueryType *
 gst_proxy_pad_do_query_type (GstPad * pad)
 {
   GstPad *target = gst_proxy_pad_get_target (pad);
@@ -436,6 +440,9 @@ static xmlNodePtr
 gst_proxy_pad_save_thyself (GstObject * object, xmlNodePtr parent)
 {
   xmlNodePtr self;
+  GstProxyPad *proxypad;
+  GstPad *pad;
+  GstPad *peer;
 
   g_return_val_if_fail (GST_IS_PROXY_PAD (object), NULL);
 
@@ -445,7 +452,29 @@ gst_proxy_pad_save_thyself (GstObject * object, xmlNodePtr parent)
   xmlNewChild (self, NULL, (xmlChar *) "parent",
       (xmlChar *) GST_OBJECT_NAME (GST_OBJECT_PARENT (object)));
 
-  /* FIXME FIXME FIXME! */
+  proxypad = GST_PROXY_PAD_CAST (object);
+  pad = GST_PAD_CAST (proxypad);
+  peer = GST_PAD_CAST (pad->peer);
+
+  if (GST_IS_PAD (pad)) {
+    if (GST_PAD_IS_SRC (pad))
+      xmlNewChild (self, NULL, (xmlChar *) "direction", (xmlChar *) "source");
+    else if (GST_PAD_IS_SINK (pad))
+      xmlNewChild (self, NULL, (xmlChar *) "direction", (xmlChar *) "sink");
+    else
+      xmlNewChild (self, NULL, (xmlChar *) "direction", (xmlChar *) "unknown");
+  } else {
+    xmlNewChild (self, NULL, (xmlChar *) "direction", (xmlChar *) "unknown");
+  }
+  if (GST_IS_PAD (peer)) {
+    gchar *content = g_strdup_printf ("%s.%s",
+        GST_OBJECT_NAME (GST_PAD_PARENT (peer)), GST_PAD_NAME (peer));
+
+    xmlNewChild (self, NULL, (xmlChar *) "peer", (xmlChar *) content);
+    g_free (content);
+  } else {
+    xmlNewChild (self, NULL, (xmlChar *) "peer", NULL);
+  }
 
   return self;
 }
@@ -481,96 +510,12 @@ G_DEFINE_TYPE (GstGhostPad, gst_ghost_pad, GST_TYPE_PROXY_PAD);
 
 static void gst_ghost_pad_dispose (GObject * object);
 
-/*
- * The parent_set and parent_unset are used to make sure that:
- * _ the internal and target are only linked when the GhostPad has a parent,
- * _ the internal and target are unlinked as soon as the GhostPad is removed
- *    from it's parent.
- */
-static void
-gst_ghost_pad_parent_set (GstObject * object, GstObject * parent)
-{
-  GstPad *gpad;
-  GstPad *target;
-  GstPad *internal;
-  GstPadLinkReturn ret;
-  GstObjectClass *klass;
-
-  gpad = GST_PAD (object);
-
-  /* internal is never NULL */
-  internal = GST_PROXY_PAD_INTERNAL (gpad);
-  target = gst_proxy_pad_get_target (gpad);
-
-  if (target) {
-    if (GST_PAD_IS_SRC (internal))
-      ret = gst_pad_link (internal, target);
-    else
-      ret = gst_pad_link (target, internal);
-
-    gst_object_unref (target);
-
-    if (ret != GST_PAD_LINK_OK)
-      goto link_failed;
-  }
-
-  klass = GST_OBJECT_CLASS (gst_ghost_pad_parent_class);
-
-  if (klass->parent_set)
-    klass->parent_set (object, parent);
-
-  return;
-
-  /* ERRORS */
-link_failed:
-  {
-    /* a warning is all we can do */
-    GST_WARNING_OBJECT (gpad, "could not link internal ghostpad %s:%s",
-        GST_DEBUG_PAD_NAME (target));
-    g_warning ("could not link internal ghostpad %s:%s",
-        GST_DEBUG_PAD_NAME (target));
-    return;
-  }
-}
-
-static void
-gst_ghost_pad_parent_unset (GstObject * object, GstObject * parent)
-{
-  GstPad *gpad;
-  GstPad *target;
-  GstPad *internal;
-  GstObjectClass *klass;
-
-  gpad = GST_PAD (object);
-  internal = GST_PROXY_PAD_INTERNAL (gpad);
-  target = gst_proxy_pad_get_target (gpad);
-
-  if (target) {
-    if (GST_PAD_IS_SRC (internal))
-      gst_pad_unlink (internal, target);
-    else
-      gst_pad_unlink (target, internal);
-
-    gst_object_unref (target);
-  }
-
-  klass = GST_OBJECT_CLASS (gst_ghost_pad_parent_class);
-
-  if (klass->parent_unset)
-    klass->parent_unset (object, parent);
-}
-
-
 static void
 gst_ghost_pad_class_init (GstGhostPadClass * klass)
 {
   GObjectClass *gobject_class = (GObjectClass *) klass;
-  GstObjectClass *gstobject_class = (GstObjectClass *) klass;
 
   gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_ghost_pad_dispose);
-  gstobject_class->parent_set = GST_DEBUG_FUNCPTR (gst_ghost_pad_parent_set);
-  gstobject_class->parent_unset =
-      GST_DEBUG_FUNCPTR (gst_ghost_pad_parent_unset);
 }
 
 /* see gstghostpad design docs */
@@ -580,17 +525,13 @@ gst_ghost_pad_internal_do_activate_push (GstPad * pad, gboolean active)
   gboolean ret;
   GstPad *other;
 
-  GST_LOG_OBJECT (pad, "%sactivate push on %s:%s", (active ? "" : "de"),
-      GST_DEBUG_PAD_NAME (pad));
+  GST_LOG_OBJECT (pad, "%sactivate push on %s:%s, we're ok",
+      (active ? "" : "de"), GST_DEBUG_PAD_NAME (pad));
 
-  if (GST_PAD_DIRECTION (pad) == GST_PAD_SINK) {
-    other = GST_PROXY_PAD_INTERNAL (pad);
-    ret = gst_pad_activate_push (other, active);
-  } else if (G_LIKELY ((other = gst_pad_get_peer (pad)))) {
-    ret = gst_pad_activate_push (other, active);
-    gst_object_unref (other);
-  } else
-    ret = FALSE;
+  /* in both cases (SRC and SINK) we activate just the internal pad. The targets
+   * will be activated later (or already in case of a ghost sinkpad). */
+  other = GST_PROXY_PAD_INTERNAL (pad);
+  ret = gst_pad_activate_push (other, active);
 
   return ret;
 }
@@ -605,13 +546,25 @@ gst_ghost_pad_internal_do_activate_pull (GstPad * pad, gboolean active)
       GST_DEBUG_PAD_NAME (pad));
 
   if (GST_PAD_DIRECTION (pad) == GST_PAD_SRC) {
+    /* we are activated in pull mode by our peer element, which is a sinkpad
+     * that wants to operate in pull mode. This activation has to propagate
+     * upstream throught the pipeline. We call the internal activation function,
+     * which will trigger gst_ghost_pad_do_activate_pull, which propagates even
+     * further upstream */
+    GST_LOG_OBJECT (pad, "pad is src, activate internal");
     other = GST_PROXY_PAD_INTERNAL (pad);
     ret = gst_pad_activate_pull (other, active);
   } else if (G_LIKELY ((other = gst_pad_get_peer (pad)))) {
+    /* We are SINK, the ghostpad is SRC, we propagate the activation upstream
+     * since we hold a pointer to the upstream peer. */
+    GST_LOG_OBJECT (pad, "activating peer");
     ret = gst_pad_activate_pull (other, active);
     gst_object_unref (other);
-  } else
+  } else {
+    /* this is failure, we can't activate pull if there is no peer */
+    GST_LOG_OBJECT (pad, "not src and no peer, failing");
     ret = FALSE;
+  }
 
   return ret;
 }
@@ -622,14 +575,12 @@ gst_ghost_pad_do_activate_push (GstPad * pad, gboolean active)
   gboolean ret;
   GstPad *other;
 
-  GST_LOG_OBJECT (pad, "%sactivate push on %s:%s", (active ? "" : "de"),
-      GST_DEBUG_PAD_NAME (pad));
+  GST_LOG_OBJECT (pad, "%sactivate push on %s:%s, proxy internal",
+      (active ? "" : "de"), GST_DEBUG_PAD_NAME (pad));
 
-  if (GST_PAD_DIRECTION (pad) == GST_PAD_SINK) {
-    other = GST_PROXY_PAD_INTERNAL (pad);
-    ret = gst_pad_activate_push (other, active);
-  } else
-    ret = TRUE;
+  /* just activate the internal pad */
+  other = GST_PROXY_PAD_INTERNAL (pad);
+  ret = gst_pad_activate_push (other, active);
 
   return ret;
 }
@@ -644,13 +595,23 @@ gst_ghost_pad_do_activate_pull (GstPad * pad, gboolean active)
       GST_DEBUG_PAD_NAME (pad));
 
   if (GST_PAD_DIRECTION (pad) == GST_PAD_SRC) {
+    /* the ghostpad is SRC and activated in pull mode by its peer, call the
+     * activation function of the internal pad to propagate the activation
+     * upstream */
+    GST_LOG_OBJECT (pad, "pad is src, activate internal");
     other = GST_PROXY_PAD_INTERNAL (pad);
     ret = gst_pad_activate_pull (other, active);
   } else if (G_LIKELY ((other = gst_pad_get_peer (pad)))) {
+    /* We are SINK and activated by the internal pad, propagate activation
+     * upstream because we hold a ref to the upstream peer */
+    GST_LOG_OBJECT (pad, "activating peer");
     ret = gst_pad_activate_pull (other, active);
     gst_object_unref (other);
-  } else
+  } else {
+    /* no peer, we fail */
+    GST_LOG_OBJECT (pad, "pad not src and no peer, failing");
     ret = FALSE;
+  }
 
   return ret;
 }
@@ -669,7 +630,7 @@ gst_ghost_pad_do_link (GstPad * pad, GstPad * peer)
 
   ret = GST_PAD_LINK_OK;
   /* if we are a source pad, we should call the peer link function
-   * if the peer has one */
+   * if the peer has one, see design docs. */
   if (GST_PAD_IS_SRC (pad)) {
     if (GST_PAD_LINKFUNC (peer)) {
       ret = GST_PAD_LINKFUNC (peer) (peer, pad);
@@ -773,8 +734,7 @@ gst_ghost_pad_dispose (GObject * object)
   GST_PROXY_PAD_INTERNAL (internal) = NULL;
 
   /* disposes of the internal pad, since the ghostpad is the only possible object
-   * that has a refcount on the internal pad.
-   */
+   * that has a refcount on the internal pad. */
   gst_object_unparent (GST_OBJECT_CAST (internal));
 
   GST_PROXY_UNLOCK (pad);
@@ -831,7 +791,7 @@ gst_ghost_pad_new_full (const gchar * name, GstPadDirection dir,
         g_object_new (GST_TYPE_PROXY_PAD, "name", NULL,
         "direction", otherdir, NULL);
   }
-  /* GST_PAD_UNSET_FLUSHING (internal); */
+  GST_PAD_UNSET_FLUSHING (internal);
 
   /* Set directional padfunctions for internal pad */
   if (dir == GST_PAD_SRC) {
@@ -1067,14 +1027,14 @@ gst_ghost_pad_get_target (GstGhostPad * gpad)
  * Set the new target of the ghostpad @gpad. Any existing target
  * is unlinked and links to the new target are established.
  *
- * Returns: TRUE if the new target could be set, FALSE otherwise.
+ * Returns: TRUE if the new target could be set. This function can return FALSE
+ * when the internal pads could not be linked.
  */
 gboolean
 gst_ghost_pad_set_target (GstGhostPad * gpad, GstPad * newtarget)
 {
   GstPad *internal;
   GstPad *oldtarget;
-  GstObject *parent;
   gboolean result;
   GstPadLinkReturn lret;
 
@@ -1099,25 +1059,16 @@ gst_ghost_pad_set_target (GstGhostPad * gpad, GstPad * newtarget)
   result = gst_proxy_pad_set_target_unlocked (GST_PAD_CAST (gpad), newtarget);
 
   if (result && newtarget) {
-    /* and link to internal pad if we are not unparent-ed */
-    if ((parent = gst_object_get_parent (GST_OBJECT (gpad)))) {
-      gst_object_unref (parent);
+    /* and link to internal pad */
+    GST_DEBUG_OBJECT (gpad, "connecting internal pad to target");
 
-      GST_DEBUG_OBJECT (gpad,
-          "GhostPad has parent, connecting internal pad to target");
+    if (GST_PAD_IS_SRC (internal))
+      lret = gst_pad_link (internal, newtarget);
+    else
+      lret = gst_pad_link (newtarget, internal);
 
-      if (GST_PAD_IS_SRC (internal))
-        lret = gst_pad_link (internal, newtarget);
-      else
-        lret = gst_pad_link (newtarget, internal);
-
-      if (lret != GST_PAD_LINK_OK)
-        goto link_failed;
-    } else {
-      /* we need to connect the internal pad once we have a parent */
-      GST_DEBUG_OBJECT (gpad,
-          "GhostPad doesn't have a parent, will connect internal pad later");
-    }
+    if (lret != GST_PAD_LINK_OK)
+      goto link_failed;
   }
   GST_PROXY_UNLOCK (gpad);
 
