@@ -205,7 +205,8 @@ gst_object_class_init (GstObjectClass * klass)
 
   g_object_class_install_property (gobject_class, ARG_NAME,
       g_param_spec_string ("name", "Name", "The name of the object",
-          NULL, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+          NULL,
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
 
   /**
    * GstObject::parent-set:
@@ -304,7 +305,7 @@ gst_object_init (GTypeInstance * instance, gpointer g_class)
  * gst_object_ref:
  * @object: a #GstObject to reference
  *
- * Increments the refence count on @object. This function
+ * Increments the reference count on @object. This function
  * does not take the lock on @object because it relies on
  * atomic refcounting.
  *
@@ -333,7 +334,7 @@ gst_object_ref (gpointer object)
  * gst_object_unref:
  * @object: a #GstObject to unreference
  *
- * Decrements the refence count on @object.  If reference count hits
+ * Decrements the reference count on @object.  If reference count hits
  * zero, destroy @object. This function does not take the lock
  * on @object as it relies on atomic refcounting.
  *
@@ -517,23 +518,14 @@ gst_object_dispatch_properties_changed (GObject * object,
   /* now let the parent dispatch those, too */
   parent = gst_object_get_parent (gst_object);
   while (parent) {
-    /* for debugging ... */
-    gchar *parent_name = gst_object_get_name (parent);
-
-#ifndef GST_DISABLE_GST_DEBUG
-    gchar *debug_parent_name = GST_STR_NULL (parent_name);
-#endif
-
-    /* need own category? */
     for (i = 0; i < n_pspecs; i++) {
-      GST_CAT_LOG (GST_CAT_EVENT, "deep notification from %s to %s (%s)",
-          debug_name, debug_parent_name, pspecs[i]->name);
+      GST_LOG_OBJECT (parent, "deep notification from %s (%s)",
+          debug_name, pspecs[i]->name);
 
       g_signal_emit (parent, gst_object_signals[DEEP_NOTIFY],
           g_quark_from_string (pspecs[i]->name), GST_OBJECT_CAST (object),
           pspecs[i]);
     }
-    g_free (parent_name);
 
     old_parent = parent;
     parent = gst_object_get_parent (old_parent);
