@@ -64,12 +64,14 @@
 #include <errno.h>
 
 #include "gst_private.h"
+#include "glib-compat-private.h"
 
 #include <gst/gst.h>
 
 #define GST_CAT_DEFAULT GST_CAT_PLUGIN_LOADING
 
-static GList *_gst_plugin_static = NULL;
+static guint _num_static_plugins;       /* 0    */
+static GstPluginDesc *_static_plugins;  /* NULL */
 static gboolean _gst_plugin_inited;
 
 /* static variables for segfault handling of plugin loading */
@@ -105,7 +107,7 @@ static const gchar *valid_licenses[] = {
 };
 
 static GstPlugin *gst_plugin_register_func (GstPlugin * plugin,
-    GstPluginDesc * desc);
+    const GstPluginDesc * desc);
 static void gst_plugin_desc_copy (GstPluginDesc * dest,
     const GstPluginDesc * src);
 static void gst_plugin_desc_free (GstPluginDesc * desc);
@@ -155,6 +157,7 @@ gst_plugin_error_quark (void)
   return quark;
 }
 
+#ifndef GST_REMOVE_DEPRECATED
 /* this function can be called in the GCC constructor extension, before
  * the _gst_plugin_initialize() was called. In that case, we store the
  * plugin description in a list to initialize it when we open the main
@@ -164,33 +167,111 @@ gst_plugin_error_quark (void)
 void
 _gst_plugin_register_static (GstPluginDesc * desc)
 {
-  if (!_gst_plugin_inited) {
-    if (GST_CAT_DEFAULT)
-      GST_LOG ("queueing static plugin \"%s\" for loading later on",
-          desc->name);
-    _gst_plugin_static = g_list_prepend (_gst_plugin_static, desc);
-  } else {
-    GstPlugin *plugin;
+  g_return_if_fail (desc != NULL);
 
-    if (GST_CAT_DEFAULT)
-      GST_LOG ("attempting to load static plugin \"%s\" now...", desc->name);
-    plugin = g_object_new (GST_TYPE_PLUGIN, NULL);
-    if (gst_plugin_register_func (plugin, desc)) {
-      if (GST_CAT_DEFAULT)
-        GST_INFO ("loaded static plugin \"%s\"", desc->name);
-      gst_default_registry_add_plugin (plugin);
-    }
+  if (!_gst_plugin_inited) {
+    /* We can't use any GLib functions here, since g_thread_init hasn't been
+     * called yet, and we can't call it here either, or programs that don't
+     * guard their g_thread_init calls in main() will just abort */
+    ++_num_static_plugins;
+    _static_plugins =
+        realloc (_static_plugins, _num_static_plugins * sizeof (GstPluginDesc));
+    /* assume strings in the GstPluginDesc are static const or live forever */
+    _static_plugins[_num_static_plugins - 1] = *desc;
+  } else {
+    gst_plugin_register_static (desc->major_version, desc->minor_version,
+        desc->name, desc->description, desc->plugin_init, desc->version,
+        desc->license, desc->source, desc->package, desc->origin);
   }
 }
+#endif
+
+/**
+ * gst_plugin_register_static:
+ * @major_version: the major version number of the GStreamer core that the
+ *     plugin was compiled for, you can just use GST_VERSION_MAJOR here
+ * @minor_version: the minor version number of the GStreamer core that the
+ *     plugin was compiled for, you can just use GST_VERSION_MINOR here
+ * @name: a unique name of the plugin (ideally prefixed with an application- or
+ *     library-specific namespace prefix in order to avoid name conflicts in
+ *     case a similar plugin with the same name ever gets added to GStreamer)
+ * @description: description of the plugin
+ * @init_func: pointer to the init function of this plugin.
+ * @version: version string of the plugin
+ * @license: effective license of plugin. Must be one of the approved licenses
+ *     (see #GstPluginDesc above) or the plugin will not be registered.
+ * @source: source module plugin belongs to
+ * @package: shipped package plugin belongs to
+ * @origin: URL to provider of plugin
+ *
+ * Registers a static plugin, ie. a plugin which is private to an application
+ * or library and contained within the application or library (as opposed to
+ * being shipped as a separate module file).
+ *
+ * You must make sure that GStreamer has been initialised (with gst_init() or
+ * via gst_init_get_option_group()) before calling this function.
+ *
+ * Returns: TRUE if the plugin was registered correctly, otherwise FALSE.
+ *
+ * Since: 0.10.16
+ */
+gboolean
+gst_plugin_register_static (gint major_version, gint minor_version,
+    const gchar * name, gchar * description, GstPluginInitFunc init_func,
+    const gchar * version, const gchar * license, const gchar * source,
+    const gchar * package, const gchar * origin)
+{
+  GstPluginDesc desc = { major_version, minor_version, name, description,
+    init_func, version, license, source, package, origin,
+  };
+  GstPlugin *plugin;
+  gboolean res = FALSE;
+
+  g_return_val_if_fail (name != NULL, FALSE);
+  g_return_val_if_fail (description != NULL, FALSE);
+  g_return_val_if_fail (init_func != NULL, FALSE);
+  g_return_val_if_fail (version != NULL, FALSE);
+  g_return_val_if_fail (license != NULL, FALSE);
+  g_return_val_if_fail (source != NULL, FALSE);
+  g_return_val_if_fail (package != NULL, FALSE);
+  g_return_val_if_fail (origin != NULL, FALSE);
+
+  /* make sure gst_init() has been called */
+  g_return_val_if_fail (_gst_plugin_inited != FALSE, FALSE);
+
+  GST_LOG ("attempting to load static plugin \"%s\" now...", name);
+  plugin = g_object_new (GST_TYPE_PLUGIN, NULL);
+  if (gst_plugin_register_func (plugin, &desc) != NULL) {
+    GST_INFO ("registered static plugin \"%s\"", name);
+    res = gst_default_registry_add_plugin (plugin);
+    GST_INFO ("added static plugin \"%s\", result: %d", name, res);
+  }
+  return res;
+}
 
 void
 _gst_plugin_initialize (void)
 {
+  guint i;
+
   _gst_plugin_inited = TRUE;
 
   /* now register all static plugins */
-  g_list_foreach (_gst_plugin_static, (GFunc) _gst_plugin_register_static,
-      NULL);
+  GST_INFO ("registering %u static plugins", _num_static_plugins);
+  for (i = 0; i < _num_static_plugins; ++i) {
+    gst_plugin_register_static (_static_plugins[i].major_version,
+        _static_plugins[i].minor_version, _static_plugins[i].name,
+        _static_plugins[i].description, _static_plugins[i].plugin_init,
+        _static_plugins[i].version, _static_plugins[i].license,
+        _static_plugins[i].source, _static_plugins[i].package,
+        _static_plugins[i].origin);
+  }
+
+  if (_static_plugins) {
+    free (_static_plugins);
+    _static_plugins = NULL;
+    _num_static_plugins = 0;
+  }
 }
 
 /* this function could be extended to check if the plugin license matches the
@@ -224,7 +305,7 @@ gst_plugin_check_version (gint major, gint minor)
 }
 
 static GstPlugin *
-gst_plugin_register_func (GstPlugin * plugin, GstPluginDesc * desc)
+gst_plugin_register_func (GstPlugin * plugin, const GstPluginDesc * desc)
 {
   if (!gst_plugin_check_version (desc->major_version, desc->minor_version)) {
     if (GST_CAT_DEFAULT)
