@@ -56,12 +56,6 @@
 #include "config.h"
 #endif
 
-#ifndef _MSC_VER
-#define _GNU_SOURCE 1
-#include <sys/poll.h>
-#include <sys/time.h>
-#endif
-
 #include <sys/types.h>
 
 #ifdef HAVE_UNISTD_H
@@ -77,6 +71,9 @@
 #include <winsock2.h>
 #define EINPROGRESS WSAEINPROGRESS
 #else
+#define _GNU_SOURCE 1
+#include <sys/poll.h>
+#include <sys/time.h>
 #include <sys/socket.h>
 #endif
 
@@ -462,7 +459,7 @@ gst_poll_new (gboolean controllable)
 {
   GstPoll *nset;
 
-  nset = g_new0 (GstPoll, 1);
+  nset = g_slice_new0 (GstPoll);
   nset->lock = g_mutex_new ();
 #ifndef G_OS_WIN32
   nset->mode = GST_POLL_MODE_AUTO;
@@ -530,7 +527,7 @@ gst_poll_free (GstPoll * set)
   g_array_free (set->active_fds, TRUE);
   g_array_free (set->fds, TRUE);
   g_mutex_free (set->lock);
-  g_free (set);
+  g_slice_free (GstPoll, set);
 }
 
 /**
