@@ -83,14 +83,26 @@
  * Last reviewed on 2007-05-17 (0.10.13)
  */
 
-static GstSegment *
+/**
+ * gst_segment_copy:
+ * @segment: a #GstSegment
+ *
+ * Create a copy of given @segment.
+ *
+ * Returns: a new #GstSegment, free with gst_segment_free().
+ *
+ * Since: 0.10.20
+ */
+GstSegment *
 gst_segment_copy (GstSegment * segment)
 {
   GstSegment *result = NULL;
 
   if (segment) {
-    result = gst_segment_new ();
-    memcpy (result, segment, sizeof (GstSegment));
+    /* we do not use g_slice_dup or g_slice_copy here because those were
+     * added in GLib 2.14 and we require only >= 2.12 */
+    result = g_slice_new (GstSegment);
+    *result = *segment;
   }
   return result;
 }
@@ -121,7 +133,7 @@ gst_segment_new (void)
 {
   GstSegment *result;
 
-  result = g_new0 (GstSegment, 1);
+  result = g_slice_new0 (GstSegment);
   gst_segment_init (result, GST_FORMAT_UNDEFINED);
 
   return result;
@@ -136,7 +148,7 @@ gst_segment_new (void)
 void
 gst_segment_free (GstSegment * segment)
 {
-  g_free (segment);
+  g_slice_free (GstSegment, segment);
 }
 
 /**
@@ -274,15 +286,13 @@ gst_segment_set_seek (GstSegment * segment, gdouble rate,
 
   if (G_UNLIKELY (segment->format == GST_FORMAT_UNDEFINED))
     segment->format = format;
-  else
-    g_return_if_fail (segment->format == format);
 
   update_start = update_stop = TRUE;
 
-  /* start is never invalid */
+  /* segment->start is never invalid */
   switch (start_type) {
     case GST_SEEK_TYPE_NONE:
-      /* no update to segment */
+      /* no update to segment, take previous start */
       start = segment->start;
       update_start = FALSE;
       break;
@@ -290,13 +300,17 @@ gst_segment_set_seek (GstSegment * segment, gdouble rate,
       /* start holds desired position, map -1 to the start */
       if (start == -1)
         start = 0;
+      /* start must be 0 or the formats must match */
+      g_return_if_fail (start == 0 || segment->format == format);
       break;
     case GST_SEEK_TYPE_CUR:
-      /* add start to currently configure segment */
+      g_return_if_fail (start == 0 || segment->format == format);
+      /* add start to currently configured segment */
       start = segment->start + start;
       break;
     case GST_SEEK_TYPE_END:
       if (segment->duration != -1) {
+        g_return_if_fail (start == 0 || segment->format == format);
         /* add start to total length */
         start = segment->duration + start;
       } else {
@@ -319,18 +333,24 @@ gst_segment_set_seek (GstSegment * segment, gdouble rate,
       update_stop = FALSE;
       break;
     case GST_SEEK_TYPE_SET:
-      /* stop holds required value */
+      /* stop holds required value, if it's not -1, it must be of the same
+       * format as the segment. */
+      g_return_if_fail (stop == -1 || segment->format == format);
       break;
     case GST_SEEK_TYPE_CUR:
-      if (segment->stop != -1)
+      if (segment->stop != -1) {
+        /* only add compatible formats or 0 */
+        g_return_if_fail (stop == 0 || segment->format == format);
         stop = segment->stop + stop;
-      else
+      } else
         stop = -1;
       break;
     case GST_SEEK_TYPE_END:
-      if (segment->duration != -1)
+      if (segment->duration != -1) {
+        /* only add compatible formats or 0 */
+        g_return_if_fail (stop == 0 || segment->format == format);
         stop = segment->duration + stop;
-      else {
+      } else {
         stop = segment->stop;
         update_stop = FALSE;
       }
@@ -354,6 +374,9 @@ gst_segment_set_seek (GstSegment * segment, gdouble rate,
   segment->applied_rate = 1.0;
   segment->flags = flags;
   segment->start = start;
+  segment->stop = stop;
+  segment->time = start;
+
   last_stop = segment->last_stop;
   if (update_start && rate > 0.0) {
     last_stop = start;
@@ -373,11 +396,7 @@ gst_segment_set_seek (GstSegment * segment, gdouble rate,
     *update = last_stop != segment->last_stop;
 
   /* update new position */
-  if (last_stop != segment->last_stop)
-    segment->last_stop = last_stop;
-
-  segment->time = start;
-  segment->stop = stop;
+  segment->last_stop = last_stop;
 }
 
 /**
@@ -567,7 +586,9 @@ gst_segment_to_stream_time (GstSegment * segment, GstFormat format,
     /* correct for segment time */
     result += time;
   } else {
-    /* correct for segment time, clamp at 0 */
+    /* correct for segment time, clamp at 0. Streams with a negative
+     * applied_rate have timestamps between start and stop, as usual, but have
+     * the time member starting high and going backwards.  */
     if (time > result)
       result = time - result;
     else
