@@ -40,12 +40,11 @@ G_BEGIN_DECLS
  * @GST_TAG_MERGE_COUNT: the number of merge modes
  *
  * The different tag merging modes are basically replace, overwrite and append,
- * but they can be seen from two directions.
- * Given two taglists: A - the one that are supplied to
- * gst_tag_setter_merge_tags() or gst_tag_setter_add_tags() and B - the tags
- * already in the element, how are the tags merged? In the table below this is
- * shown for the cases that a tag exists in the list (A) or does not exists (!A)
- * and combination thereof.
+ * but they can be seen from two directions.  Given two taglists: (A) the tags
+ * already in the element and (B) the ones that are supplied to
+ * gst_tag_setter_merge_tags() or gst_tag_setter_add_tags(), how are these tags
+ * merged? In the table below this is shown for the cases that a tag exists in
+ * the list (A) or does not exists (!A) and combinations thereof.
  *
  * <table frame="all" colsep="1" rowsep="1">
  *   <title>merge mode</title>
@@ -173,10 +172,9 @@ typedef void (*GstTagForeachFunc) (const GstTagList * list,
  */
 typedef void (* GstTagMergeFunc) (GValue *dest, const GValue *src);
 
-/* initialize tagging system */
-void         _gst_tag_initialize   (void);
 GType        gst_tag_list_get_type (void);
 
+/* tag registration */
 void         gst_tag_register      (const gchar     * name,
                                     GstTagFlag        flag,
                                     GType             type,
@@ -345,20 +343,67 @@ gboolean     gst_tag_list_get_date_index    (const GstTagList * list,
  * GST_TAG_TITLE:
  *
  * commonly used title (string)
+ *
+ * The title as it should be displayed, e.g. 'The Doll House'
  */
 #define GST_TAG_TITLE                  "title"
+/**
+ * GST_TAG_TITLE_SORTNAME:
+ *
+ * commonly used title, as used for sorting (string)
+ *
+ * The title as it should be sorted, e.g. 'Doll House, The'
+ *
+ * Since: 0.10.15
+ */
+#define GST_TAG_TITLE_SORTNAME         "title-sortname"
 /**
  * GST_TAG_ARTIST:
  *
  * person(s) responsible for the recording (string)
+ *
+ * The artist name as it should be displayed, e.g. 'Jimi Hendrix' or
+ * 'The Guitar Heroes'
  */
 #define GST_TAG_ARTIST                 "artist"
+/**
+ * GST_TAG_ARTIST_SORTNAME:
+ *
+ * person(s) responsible for the recording, as used for sorting (string)
+ *
+ * The artist name as it should be sorted, e.g. 'Hendrix, Jimi' or
+ * 'Guitar Heroes, The'
+ *
+ * Since: 0.10.15
+ */
+/* FIXME 0.11: change to "artist-sortname" */
+#define GST_TAG_ARTIST_SORTNAME        "musicbrainz-sortname"
 /**
  * GST_TAG_ALBUM:
  *
  * album containing this data (string)
+ *
+ * The album name as it should be displayed, e.g. 'The Jazz Guitar'
  */
 #define GST_TAG_ALBUM                  "album"
+/**
+ * GST_TAG_ALBUM_SORTNAME:
+ *
+ * album containing this data, as used for sorting (string)
+ *
+ * The album name as it should be sorted, e.g. 'Jazz Guitar, The'
+ *
+ * Since: 0.10.15
+ */
+#define GST_TAG_ALBUM_SORTNAME         "album-sortname"
+/**
+ * GST_TAG_COMPOSER:
+ *
+ * person(s) who composed the recording (string)
+ *
+ * Since: 0.10.15
+ */
+#define GST_TAG_COMPOSER               "composer"
 /**
  * GST_TAG_DATE:
  *
@@ -587,13 +632,14 @@ gboolean     gst_tag_list_get_date_index    (const GstTagList * list,
 /**
  * GST_TAG_LANGUAGE_CODE:
  *
- * Language code (ISO-639-1) (string)
+ * Language code (ISO-639-1) (string) of the content
  */
 #define GST_TAG_LANGUAGE_CODE          "language-code"
 /**
  * GST_TAG_IMAGE:
  *
- * image (buffer) (buffer caps should specify the content type)
+ * image (buffer) (buffer caps should specify the content type and preferably
+ * also set "image-type" field as #GstTagImageType)
  *
  * Since: 0.10.6
  */
@@ -601,8 +647,8 @@ gboolean     gst_tag_list_get_date_index    (const GstTagList * list,
 /**
  * GST_TAG_PREVIEW_IMAGE:
  *
- * image that is meant for preview purposes (buffer)
- * (buffer caps should specify the content type)
+ * image that is meant for preview purposes, e.g. small icon-sized version
+ * (buffer) (buffer caps should specify the content type)
  *
  * Since: 0.10.7
  */
